import com.jfx.*;
import com.jfx.net.JFXServer;
import com.jfx.strategy.Strategy;
import com.jfx.strategy.StrategyRunner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

public class JFXExample extends Strategy {
    private int shift;
    public double buyPrice;
    private static final String EURUSD = "EURUSD";

    public JFXExample() {
    }

    public synchronized void init(String symbol, int period, StrategyRunner strategyRunner) throws ErrUnknownSymbol, IOException {
        super.init(symbol, period, strategyRunner);
        //
        System.out.println("Available instruments:");
        ArrayList<Instrument> instruments = getInstruments();
        for (int i = 0; i < instruments.size(); i++) {
            Instrument s =  instruments.get(i);
            System.out.println("" + s);
        }
        System.out.println("");
    }

    public void coordinate() {
        try {
            int ordersCount = ordersTotal();
            //
            if (ordersCount == 0) {
                System.out.println("ordersTotal: " + ordersCount + ", accBal=" + accountBalance());
                //
                double point = marketInfo(EURUSD, MarketInfo.MODE_POINT);
                double price = marketInfo(EURUSD, MarketInfo.MODE_ASK);
                buyPrice = price;
                int ticket = orderSend(
                        EURUSD,
                        TradeOperation.OP_BUY,
                        1,
                        buyPrice,
                        2,
                        price - 100 * point,
                        price + 100 * point,
                        "" + System.currentTimeMillis(),
                        0,
                        new Date(System.currentTimeMillis() + 60 * 60 * 1000),
                        -1
                );
                System.out.println("---------------------------------------");
                System.out.println("Buy order ticket: " + ticket);
            } else {
                if (orderSelect(0, SelectionType.SELECT_BY_POS, SelectionPool.MODE_TRADES)) {
                    int ticket = orderTicket();
                    String orderSymbol = orderSymbol();
                    double bid = marketInfo(orderSymbol, MarketInfo.MODE_BID);
                    //
                    String startTime = orderComment();
                    long _startTime = 0;
                    try {
                        _startTime = Long.parseLong(startTime);
                    } catch (NumberFormatException e) {
                    }
                    if (_startTime == 0) {
                        System.out.println("Closing order, ticket=" + ticket + ", profit=" + orderProfit() + ", symbol=" + orderSymbol + ", volume=" + orderLots());
                        if (orderType() == TradeOperation._OP_BUY) {
                            orderClose(ticket, orderLots(), bid, 2, 0);
                        } else {
                            orderClose(ticket, orderLots(), marketInfo(orderSymbol, MarketInfo.MODE_ASK), 2, 0);
                        }
                    } else {
                        long seconds = (System.currentTimeMillis() - _startTime) / 1000;
                        if (seconds % 10 == 0) {
                            if (orderProfit() > 0) {
                                System.out.println("Closing order, ticket=" + ticket + ", profit=" + orderProfit());
                                orderClose(ticket, 1, bid, 2, 0);
                            } else if (seconds > 600) {
                                System.out.println("Closing order, ticket=" + ticket + ", loss=" + orderProfit());
                                orderClose(ticket, 1, bid, 2, 0);
                            }
                        } else {
                            if (shift++ < 10000) {
                                //
                                // Load history
                                //
                                iOpen(EURUSD, Timeframe.PERIOD_H1, shift);
                                iClose(EURUSD, Timeframe.PERIOD_H1, shift);
                                iHigh(EURUSD, Timeframe.PERIOD_H1, shift);
                                iLow(EURUSD, Timeframe.PERIOD_H1, shift);
                                if (shift % 10 == 0) {
                                    System.out.println("History H1 shift: " + shift);
                                }
                            }
                        }
                    }
                }
            }
        } catch (ErrCustomIndicatorError errCustomIndicatorError) {
            errCustomIndicatorError.printStackTrace();
        } catch (ErrIntegerParameterExpected errIntegerParameterExpected) {
            errIntegerParameterExpected.printStackTrace();
        } catch (ErrInvalidFunctionParamvalue errInvalidFunctionParamvalue) {
            errInvalidFunctionParamvalue.printStackTrace();
        } catch (ErrInvalidPriceParam errInvalidPriceParam) {
            errInvalidPriceParam.printStackTrace();
        } catch (ErrInvalidTicket errInvalidTicket) {
            errInvalidTicket.printStackTrace();
        } catch (ErrUnknownSymbol errUnknownSymbol) {
            errUnknownSymbol.printStackTrace();
        } catch (ErrTradeNotAllowed errTradeNotAllowed) {
            errTradeNotAllowed.printStackTrace();
        } catch (ErrStringParameterExpected errStringParameterExpected) {
            errStringParameterExpected.printStackTrace();
        } catch (ErrShortsNotAllowed errShortsNotAllowed) {
            errShortsNotAllowed.printStackTrace();
        } catch (ErrLongsNotAllowed errLongsNotAllowed) {
            errLongsNotAllowed.printStackTrace();
        } catch (ErrNoOrderSelected errNoOrderSelected) {
            errNoOrderSelected.printStackTrace();
        } catch (ErrHistoryWillUpdated errHistoryWillUpdated) {
            System.out.println("Requested history data in updating state.");
        } catch (ErrNotEnoughMoney errNotEnoughMoney) {
            throw new RuntimeException(errNotEnoughMoney);
        } catch (ErrCommonError errCommonError) {
            throw new RuntimeException(errCommonError);
        } catch (ErrPriceChanged errPriceChanged) {
            throw new RuntimeException(errPriceChanged);
        } catch (ErrLongPositionsOnlyAllowed errLongPositionsOnlyAllowed) {
            throw new RuntimeException(errLongPositionsOnlyAllowed);
        } catch (ErrInvalidPrice errInvalidPrice) {
            throw new RuntimeException(errInvalidPrice);
        } catch (ErrServerBusy errServerBusy) {
            throw new RuntimeException(errServerBusy);
        } catch (ErrOldVersion errOldVersion) {
            throw new RuntimeException(errOldVersion);
        } catch (ErrOffQuotes errOffQuotes) {
            throw new RuntimeException(errOffQuotes);
        } catch (ErrTradeTimeout4 errTradeTimeout4) {
            throw new RuntimeException(errTradeTimeout4);
        } catch (ErrTooFrequentRequests errTooFrequentRequests) {
            throw new RuntimeException(errTooFrequentRequests);
        } catch (ErrInvalidStops errInvalidStops) {
            throw new RuntimeException(errInvalidStops);
        } catch (ErrTradeContextBusy errTradeContextBusy) {
            throw new RuntimeException(errTradeContextBusy);
        } catch (ErrNoConnection errNoConnection) {
            throw new RuntimeException(errNoConnection);
        } catch (ErrInvalidAccount errInvalidAccount) {
            throw new RuntimeException(errInvalidAccount);
        } catch (ErrTooManyRequests errTooManyRequests) {
            throw new RuntimeException(errTooManyRequests);
        } catch (ErrTradeTimeout2 errTradeTimeout2) {
            throw new RuntimeException(errTradeTimeout2);
        } catch (ErrInvalidTradeParameters errInvalidTradeParameters) {
            throw new RuntimeException(errInvalidTradeParameters);
        } catch (ErrTradeTimeout3 errTradeTimeout3) {
            throw new RuntimeException(errTradeTimeout3);
        } catch (ErrOrderLocked errOrderLocked) {
            throw new RuntimeException(errOrderLocked);
        } catch (ErrMarketClosed errMarketClosed) {
            throw new RuntimeException(errMarketClosed);
        } catch (ErrRequote errRequote) {
            throw new RuntimeException(errRequote);
        } catch (ErrInvalidTradeVolume errInvalidTradeVolume) {
            throw new RuntimeException(errInvalidTradeVolume);
        } catch (ErrAccountDisabled errAccountDisabled) {
            throw new RuntimeException(errAccountDisabled);
        } catch (ErrTradeTimeout errTradeTimeout) {
            throw new RuntimeException(errTradeTimeout);
        } catch (ErrTradeTooManyOrders errTradeTooManyOrders) {
            throw new RuntimeException(errTradeTooManyOrders);
        } catch (ErrTradeModifyDenied errTradeModifyDenied) {
            throw new RuntimeException(errTradeModifyDenied);
        } catch (ErrTradeExpirationDenied errTradeExpirationDenied) {
            throw new RuntimeException(errTradeExpirationDenied);
        } catch (ErrTradeDisabled errTradeDisabled) {
            throw new RuntimeException(errTradeDisabled);
        }
    }

    public static void main(String[] args) throws IOException, InterruptedException {
        JFXExample jfxExample = new JFXExample();
        //
        System.out.println("You may kill terminal.exe to test reconnection...");
        jfxExample.setReconnect(true);
        //
        jfxExample.connect("127.0.0.1", 7788, Broker.AlpariUK_Demo, "1107623", "w4ylosg");
        //
        System.out.println("Working...");
        Thread.sleep(120000);
        //
        System.out.println("MT4 Terminal client disconnect...");
        jfxExample.disconnect();
        //
        System.out.println("Stopping JFXServer...");
        JFXServer.stop();
        //
        System.out.println("Exit.");
    }
}
